/*
 * ModeShape (http://www.modeshape.org)
 * See the COPYRIGHT.txt file distributed with this work for information
 * regarding copyright ownership.  Some portions may be licensed
 * to Red Hat, Inc. under one or more contributor license agreements.
 * See the AUTHORS.txt file in the distribution for a full listing of 
 * individual contributors. 
 *
 * ModeShape is free software. Unless otherwise indicated, all code in ModeShape
 * is licensed to you under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * ModeShape is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.modeshape.web.jcr.spi;

import java.util.Collections;
import java.util.Set;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import org.modeshape.jcr.api.SecurityContextCredentials;
import org.modeshape.web.jcr.ServletSecurityContext;

/**
 * Repository provider backed by a JCR {@link Repository} implementation found in JNDI.
 * <p>
 * The provider uses two servlet context parameters: {@code org.modeshape.web.jcr.JNDI_NAME} and {@code
 * org.modeshape.web.jcr.JCR_REPOSITORY_NAME}.
 * </p>
 * <p>
 * This class is thread-safe.
 * </p>
 * 
 * @see RepositoryProvider
 * @see Class#getResourceAsStream(String)
 */
public class JndiRepositoryProvider implements RepositoryProvider {

    public static final String JCR_JNDI_NAME = "org.modeshape.web.jcr.JNDI_NAME";
    public static final String JCR_REPOSITORY_NAME = "org.modeshape.web.jcr.JCR_REPOSITORY_NAME";

    private String repositoryJndiName;
    private String repositoryName;

    public JndiRepositoryProvider() {
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.modeshape.web.jcr.spi.RepositoryProvider#getJcrRepositoryNames()
     */
    public Set<String> getJcrRepositoryNames() {
        return Collections.singleton(this.repositoryName);
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.modeshape.web.jcr.spi.RepositoryProvider#startup(javax.servlet.ServletContext)
     */
    public void startup( ServletContext context ) {
        this.repositoryJndiName = context.getInitParameter(JCR_JNDI_NAME);
        this.repositoryName = context.getInitParameter(JCR_REPOSITORY_NAME);
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.modeshape.web.jcr.spi.RepositoryProvider#shutdown()
     */
    public void shutdown() {
        // do nothing, as we don't own the JCR repository instance ...
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.modeshape.web.jcr.spi.RepositoryProvider#getSession(javax.servlet.http.HttpServletRequest, java.lang.String,
     *      java.lang.String)
     */
    public Session getSession( HttpServletRequest request,
                               String repositoryName,
                               String workspaceName ) throws RepositoryException {
        assert request != null;

        Repository repository;

        // Look up the Repository instance in JNDI ...
        try {
            InitialContext initCtx = new InitialContext();
            Context envCtx = (Context)initCtx.lookup("java:comp/env");
            repository = (Repository)envCtx.lookup(this.repositoryJndiName);
        } catch (Exception ex) {
            throw new NoSuchRepositoryException(ex.getMessage(), ex);
        }

        // If there's no authenticated user, try an anonymous login
        if (request.getUserPrincipal() == null) {
            return repository.login(workspaceName);
        }

        return repository.login(new SecurityContextCredentials(new ServletSecurityContext(request)), workspaceName);

    }
}
